#!/home/moerman/cadbin/perl -w
# Japanese puzzle solver V1.2, C.M Moerman april 2001 - 2004
#   http://home.hccnet.nl/Kees.Moerman/perl_jap.html  mailto:moerman@iaehv.nl
#
  my $help_text = <<HELP;		# inline help text placed here
# $0 V1.2, C.M. Moerman (c) April 2001 - 2004 (c)
#
# This Perl program solves so-called Japanese puzzles. In these puzzles,
# you have to figure out which squares to color, based upon the given
# numbers of colored squares per row or column. Now, the computer can!
#
# Syntax: $0 [ -options ] <inputfile>
# Where options are:
#   -?   displays this help message (and quits)
#   -d1  turn on debug (list intermediate results)
#   <inputfile> is the textual description of the puzzle, see supplied examples
HELP
#
# Kees Moerman, April 15, 2001
#
# Change list:
#	V1.1 C.M.Moerman, Sept 23, 2001: cleaned up for web publishing
#   V1.2 C.M.Moerman, Dec 31, 2003: better input error checking
#
# To do:
#   Alternative searching in case program can't converge (in case we get stuck
#   with multiple remaining possibilities): guess and backtrack? No examples.

use strict;                             # don't forget to prefix vars with 'my'


# ----- Let' first parse the command line for options and file name -------

my $opt_d = 0;                          # default -d0 (debug options)

$help_text =~ s/^# ?//gm;               # clean up comment characters

if (!defined $ARGV[0] || $ARGV[0] eq "-?" || $ARGV[0] eq "-h"  )
{   print $help_text;                   # no argument, or '-?'or '-h' argument
    exit 1;
}

while ($ARGV[0] =~ /^-(.)(.*)/o)
{   $opt_d = $2  if ($1 eq "d");        # -d<n>: debug options
    shift;                              # on to next argument
}

my $filename = shift || die "No puzzle file specified for $0";


# ---------------- OK, have file name, let's read data ---------------------

open (FILENAME, $filename)              # open file for output
|| die "Can't open file '$filename' ($!)\n";

my ($i, @x_patterns, @y_patterns);

sub get_next                            # read line from input file
{   my $line = <FILENAME>;
    chomp $line;                        # remove end-of-line
    $line =~ s/\s*//g;                  # remove spaces
    if( $line !~ /^[0-9,]+$/ )          # only numeric data and commas?
    {
        print "Illegal characters in line $. '$line'\n";
        exit(1);
    }
    return $line;
}

### file has structure: <title><x-count>N*<x-series><y-count>N*<y-series>
#   fields are separated by a <newline> character (so multi-line file)
#   So we create two arrays, one with the column descriptions (@x_patterns),
#   and one with the row description (@y_patterns).
### Each has string/text information like "3,2,1" or "1" or even "0"

print "@{[ scalar <FILENAME> ]}";       # get/print title line from file (dirty...?)

my $xmax = get_next();                  # x patterns (running in y direction!)
for (1..$xmax) { push @x_patterns, get_next(); }

my $ymax = get_next();                  # y patterns (running in x direction!)
for (1..$ymax) { push @y_patterns, get_next(); }

# print "$xmax $ymax\n@x_patterns\n@y_patterns\n";

close FILENAME;                         # file read, no linger needed


### min_length calculates the minimum length for a row or column
#   based on the textual description e.g. "3,2,1" ( -> "XXX XX X" ) -> 8
#   Input: description (string e.g. "3,2,1")
### Output: number of positions needed using single spacing

sub min_length                          # calculate minimum length needed for row/column
{                                       # argument: column or row description e.g. "3,2,1"
    my $pattern = shift;
    my @values = split(/,/, $pattern);  # make array of values (in file separated by ',')
    my $i;
    my $min_length = $#values;          # initialise by number of gaps (min. size = 1)
    for $i (0..$#values)
        { $min_length += $values[$i]; } # add sizes of blocks
    return $min_length;
}

# print min_length("3,2,1");            # prints '8': 3 + 1(gap) + 2 + 1(gap) + 1




### make_alternatives generates (in the global variable @alternatives)
#   all possible alternatives based on a given row/column description ("3,2,1")
#   and the length of the field. It does so recursively (using make_altern_rec)
#   by placing the first element in all possible positions and then going for
#   the same process for the remainder of the pattern in the remaining space
#   (keeping in mind how much space is needed, as calculated with min_length).
#
#   Example: "3,2,1" in a field of 10: 2,1 needs minimum of 4 positions, plus space
#       so the 3 can be in 3 positions (max space in front: 10-5-3): 0,1,2 leading spaces
#       for each of these, we recurse for the "2,1" part, generating:
#       pos 0 (recurse with "2,1", "XXX ",   6) -> "XXX XX X  ", "XXX XX  X ", ....
#       pos 1 (recurse with "2,1", " XXX ",  5) -> " XXX XX X ", " XXX XX  X", " XXX  X X"
#       pos 2 (recurse with "2,1", "  XXX ", 4) -> "  XXX XX X"
#
#   Actual values are stored as integer bit masks, the intermediate textual
### representation helped during debugging. E.g. "XXX XX X  " -> 1110110100

my @alternatives;                       # global list to collect generated alternatives

sub vectorize                           # translate ' ' and 'X' pattern to bits
{   my $input = shift;                  # input: row/column like "X  XXX X"
    $input =~ tr/ X/01/;                # ' ' translates to 0, 'X' to 1
    return unpack("N", pack("B32", substr("0" x 32 . $input, -32))) # returns integer value
}

sub make_altern_rec                     # recursive helper for make_alternatives
{
    my ($pattern, $head, $maxlen) = (@_);   # remaining pattern, start so far, rem. length
    my $gap;
    if($pattern =~ /^\d+$/)             # single number left: only one block of 'XXX's
    {
        my $tail = "";
        for $gap (0..$maxlen-$pattern)  # number of possible placements of block
        {                               # so let's generate all of them
            $tail = (" " x $gap) . ("X" x $pattern) . (" " x ($maxlen-($pattern+$gap)));
            push @alternatives, vectorize("$head$tail");    # add to list
        }
    }
    else                                # we have multiple numbers like "3,2,1"
    {                                   # so tackle recursively
        my ($first, $last) = $pattern =~ /^(\d+?),(.*)$/;   # split off first number
        for $gap (0..$maxlen - (min_length($last)+$first+1) )
        {
            my $mid = (" " x $gap) . ("X" x $first) . " ";
            make_altern_rec($last, $head . $mid, $maxlen-length($mid));
        }
    }
}

sub make_alternatives                   # recursively generate alternatives
{   my ($pattern, $maxlen) = (@_);      # e.g. "3,2,1", 10
    return (0) if($pattern =~ /^0$/);   # "0" -> no 'X'es
    
    @alternatives = ();                 # start with empty list
    make_altern_rec($pattern, "", $maxlen);
    return (@alternatives);             # return list of them
}



### calc_masks calculate from a list of alternatives which squares are
#   always empty (over all alternatives) and which ones are always set.
#   These are the blocks we know for sure what their value will be.
#   Input: length of the pattern, list of patterns
### Putput: two values (in list): set bitmask, clear bitmask.

sub calc_masks                          # bitmasks for 'X' and ' ' elements
{   my $length = shift;                 # when combining all current alternatives. 
    my $clear  = 0;                     # I.e. check which 'X's and ' 's appear in
    my $set    = 0xFFFFFFFF;            # all alternatives so we know we are sure
    my ($i, $temp);                     # of those.
    
    for $i (@_)
    {   $set &= $i;                     # keep set   only those bits always set
        $clear |= $i;                   # keep clear only those bits always clear
    }
    $clear = (~$clear) & (0xFFFFFFFF >> (32-$length));  # invert mask: clear -> set
    return ($set, $clear);
}



### Note: we have a lot of information hanging around, actual even duplicate ones.
#   Below the declaration of the arrays holding the lists of alternatives,
#   Plus the arrays with bit masks of which squares we know for sure to be set/clear
#   Each array has an element for each column (@x_...) resp. row (@y_...)
#
#   init_set_clear gets all the set/clear bit masks for the initial configuration
#   and puts this in the global arrays keeping track of this.
#
#   The duplicates are in the bit masks: we have these organised per row, but also
### per column. The routine copy_set_clear keeps these consistent

my (@x_set, @x_clear,                   # set/clear mask arrays (array with column info)
    @x_alts,                            # alternatives (array of list per column)
    @y_set, @y_clear, @y_alts);         # same for rows

sub init_set_clear                      # create initial arrays based on raw posibilities
{                                       # remember: x patterns run in y direction
    for $i (0..$xmax-1)                 # x describes the columns
    {
        $x_alts[$i] = [make_alternatives($x_patterns[$i], $ymax)];
        ($x_set[$i], $x_clear[$i]) = calc_masks($ymax, @{$x_alts[$i]});
    }
    for $i (0..$ymax-1)                 # and y keeps the rows
    {
        $y_alts[$i] = [make_alternatives($y_patterns[$i], $xmax)];
        ($y_set[$i], $y_clear[$i]) = calc_masks($xmax, @{$y_alts[$i]});
    }
}

sub copy_set_clear                      # transpose and merge the row/column matrices
{   my ($i, $j);                        # by copying what we know for sure
    for $i (0..$xmax-1)
    {                                   # for all columns
        for $j (0..$ymax-1)             # for all rows
        {                               # copy 'known' bits from column to row
            $y_set[$j]   |= ( ($x_set[$i]   & (1<<$ymax-($j+1))) ? 1:0 ) << $xmax-($i+1);
            $y_clear[$j] |= ( ($x_clear[$i] & (1<<$ymax-($j+1))) ? 1:0 ) << $xmax-($i+1);
        }
    }
    for $i (0..$ymax-1)                 # and also in the other direction, row to column
    {
        for $j (0..$xmax-1)             # wow, this is tricky.......
        {                               # how do all these indices run?
            $x_set[$j]   |= ( ($y_set[$i]   & (1<<$xmax-($j+1))) ? 1:0 ) << $ymax-($i+1);
            $x_clear[$j] |= ( ($y_clear[$i] & (1<<$xmax-($j+1))) ? 1:0 ) << $ymax-($i+1);
        }
    }
}



### So now we have all the book-keeping functions, time to do the actual work.
#   iterate_x will work over the columns, pruning the list of alternatives
#   based on set/clear masks of the columns versus rows to see if we can
#   eliminate alternatives from the list.
#   Later, in iterate_y we do the same for the rows versus the column info
### This will be alternated till we find the solution

sub iterate_x                           # iterate over the columns
{   my ($count, @temp);
    for $i (0..$xmax-1)
    {
        @{$x_alts[$i]} =                # receive reduces list of alternatives:
          map                           #   perform for all current alternatives:
            {                           #     check if consistent with rows
                @temp =
                (  (($x_set[$i]   &  $_) == $x_set[$i])   &&    # check 'X's
                   (($x_clear[$i] & ~$_) == $x_clear[$i])       # check ' 's
                )?($_):();              # if consistent, return element, else nothing
                (@temp);
            }
            @{$x_alts[$i]};             # input alternatives list to 'map' operation above
            
        ($x_set[$i], $x_clear[$i]) = calc_masks($ymax, @{$x_alts[$i]}); # remake sets
    }
}

sub iterate_y                           # iterate over the rows
{   my ($count, @temp);
    for $i (0..$ymax-1)
    {
        @{$y_alts[$i]} = map
            {   
                @temp =
                (  (($y_set[$i]   &  $_) == $y_set[$i])   &&
                   (($y_clear[$i] & ~$_) == $y_clear[$i])
                )?($_):();
                (@temp);
            }
            @{$y_alts[$i]};
        ($y_set[$i], $y_clear[$i]) = calc_masks($xmax, @{$y_alts[$i]}); # remake sets
    }
}

my $error = 0;                          # set in case no alternatives left: input error

sub count_alternatives                  # how many do we currently have left?
{   my ($i, $total, $current);

    $total = 1;
    print "x alternatives: ";           # first do the columns
    for $i (0..$xmax-1)
    {
        $current = 1+$#{$x_alts[$i]};   # count alternatives in x direction
        print "$current, ";             # i.e. how many alternatives in array
        $error = 1 if $current == 0;       # should always be at least one alternative...
        $total *= $current;             # and accumulate in total
    }
    print " x total = $total\ny alternatives: ";
    for $i (0..$ymax-1)
    {
        $current = 1+$#{$y_alts[$i]};
        print "$current, ";
        $error = 1 if $current == 0;       # should always be at least one alternative...
        $total *= $current;
    }
    print " x*y total = $total\n";
    return $total;
}

sub print_matrix                        # OK, try to print result
{   my ($i, $j, $temp, $line);          # by inspecting the bit patterns

    print "\n";                         # start at new line
    for $j (0..$ymax-1)                 # remember: x patterns run in y direction
    {   $line = "";                     # build up and print per line (in $line)
        for $i (0..$xmax-1)             # iterate per line over 0..xmax-1
        {                               # indexing into y arrays (horizontals)
            $temp = "?";                # and lsb to the left.....
            $temp = "X" if ($y_set[$j] & (1 << $xmax-($i+1)));
            $temp = "." if ($y_clear[$j] & (1 << $xmax-($i+1)));
            $line .= $temp;
        }
        print "$line\n";
    }
}

sub main                                # the actual work (an data already read in)
{
    my $previous = -1;                  # keeps number of previous alternatives
    init_set_clear();                   # get initial lists of alternatives
    copy_set_clear();                   # and transpose/copy x <-> y
    print_matrix() if $opt_d & 1;       # this is initial configuration
    my $alts = count_alternatives();    # and the sum of all alternatives

    while (!$error && ($previous != $alts)) # while converging (less alt's then before)
    {
        $previous = $alts;                  
        iterate_x();                    # iterate in x direction
        copy_set_clear();               # and make sure same in x and y
        print_matrix() if $opt_d & 1;
#       count_alternatives();
    
        iterate_y();                    # now iterate in y direction
        copy_set_clear();               # and again make consistent
        print_matrix() if $opt_d & 1;
        $alts = count_alternatives();
    }

    if ($alts == 1)                     # every line has exactly one alternative
    {
        print "\nFinal solution:\n";    # so we have a solution
    }
    elsif ($error || ($alts == 0))      # error, or not even one alternative
    {
        print "\nProbably error in input, no solution exists\n";
    }
    else                                # multiple alternatives?
    {
        print "\nCan't converge:\n" if $alts != $xmax+$ymax;
    }
    print_matrix();
}

main();

#end of perl script

